=== modified file 'NEWS'
--- NEWS	2006-08-31 18:18:09 +0000
+++ NEWS	2006-08-31 21:34:20 +0000
@@ -3,6 +3,10 @@
   IMPROVEMENTS:
 
   BUG FIXES:
+
+    * Handle boundary="" lines properly to allow access through a Squid proxy.
+      (John Arbash Meinel, #57723)
+
     * revert now removes newly-added directories (Aaron Bentley, #54172)
 
     * ``bzr upgrade sftp://`` shouldn't fail to upgrade v6 branches if there 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-08-31 18:18:09 +0000
+++ bzrlib/errors.py	2006-08-31 21:34:20 +0000
@@ -795,11 +795,13 @@
 
 
 class InvalidRange(TransportError):
-    """Invalid range access."""
+    """Invalid range access in %(path)s at %(offset)s."""
     
     def __init__(self, path, offset):
         TransportError.__init__(self, ("Invalid range access in %s at %d"
                                        % (path, offset)))
+        self.path = path
+        self.offset = offset
 
 
 class InvalidHttpResponse(TransportError):

=== modified file 'bzrlib/tests/test_http_response.py'
--- bzrlib/tests/test_http_response.py	2006-07-19 14:17:49 +0000
+++ bzrlib/tests/test_http_response.py	2006-09-01 17:05:01 +0000
@@ -146,12 +146,20 @@
 
     def test_content_type_re(self):
         self.regex = response.HttpMultipartRangeResponse._CONTENT_TYPE_RE
-        self.assertRegexMatches(('xxyyzz',),
+        self.assertRegexMatches(('', 'xxyyzz'),
                                 'multipart/byteranges; boundary = xxyyzz')
-        self.assertRegexMatches(('xxyyzz',),
+        self.assertRegexMatches(('', 'xxyyzz'),
                                 'multipart/byteranges;boundary=xxyyzz')
-        self.assertRegexMatches(('xx yy zz',),
+        self.assertRegexMatches(('', 'xx yy zz'),
                                 ' multipart/byteranges ; boundary= xx yy zz ')
+        self.assertRegexMatches(('"', 'xx yy zz'),
+                                ' multipart/byteranges ; boundary= "xx yy zz" ')
+        self.assertEqual(None,
+                         self.regex.match(
+                             ' multipart/byteranges ; boundary= "xx yy zz '))
+        self.assertEqual(None,
+                         self.regex.match(
+                             ' multipart/byteranges ; boundary= xx yy zz" '))
         self.assertEqual(None,
                 self.regex.match('multipart byteranges;boundary=xx'))
 
@@ -389,7 +397,38 @@
 mbp@sourcefrog.net-20050314025737-55eb441f430ab4ba
 mbp@sourcefrog.net-20050314025901-d74aa93bb7ee8f62
 mbp@source\r
---418470f848b63279b--\r\n'
+--418470f848b63279b--\r
+""")
+
+_multipart_squid_range_response = (206, """HTTP/1.0 206 Partial Content\r
+Date: Thu, 31 Aug 2006 21:16:22 GMT\r
+Server: Apache/2.2.2 (Unix) DAV/2\r
+Last-Modified: Thu, 31 Aug 2006 17:57:06 GMT\r
+Accept-Ranges: bytes\r
+Content-Type: multipart/byteranges; boundary="squid/2.5.STABLE12:C99323425AD4FE26F726261FA6C24196"\r
+Content-Length: 598\r
+X-Cache: MISS from localhost.localdomain\r
+X-Cache-Lookup: HIT from localhost.localdomain:3128\r
+Proxy-Connection: keep-alive\r
+\r
+""",
+"""\r
+--squid/2.5.STABLE12:C99323425AD4FE26F726261FA6C24196\r
+Content-Type: text/plain\r
+Content-Range: bytes 0-99/18672\r
+\r
+# bzr knit index 8
+
+scott@netsplit.com-20050708230047-47c7868f276b939f fulltext 0 863  :
+scott@netsp\r
+--squid/2.5.STABLE12:C99323425AD4FE26F726261FA6C24196\r
+Content-Type: text/plain\r
+Content-Range: bytes 300-499/18672\r
+\r
+com-20050708231537-2b124b835395399a :
+scott@netsplit.com-20050820234126-551311dbb7435b51 line-delta 1803 479 .scott@netsplit.com-20050820232911-dc4322a084eadf7e :
+scott@netsplit.com-20050821213706-c86\r
+--squid/2.5.STABLE12:C99323425AD4FE26F726261FA6C24196--\r
 """)
 
 
@@ -476,6 +515,14 @@
         self.check_header('Content-Type',
                           'multipart/byteranges; boundary=418470f848b63279b')
 
+    def test_multi_squid_range(self):
+        self.use_response(_multipart_squid_range_response)
+
+        self.check_header('Content-Length', '598')
+        self.check_header('Content-Type',
+                          'multipart/byteranges; '\
+                          'boundary="squid/2.5.STABLE12:C99323425AD4FE26F726261FA6C24196"')
+
     def test_redirect(self):
         """We default to returning the last group of headers in the file."""
         self.use_response(_redirect_response)
@@ -561,6 +608,17 @@
         out.seek(1000)
         out.read(1050)
 
+    def test_multi_squid_range(self):
+        out = self.get_response(_multipart_squid_range_response)
+        self.assertIsInstance(out, response.HttpMultipartRangeResponse)
+
+        # Just make sure we can read the right contents
+        out.seek(0)
+        out.read(100)
+
+        out.seek(300)
+        out.read(200)
+
     def test_invalid_response(self):
         self.assertRaises(errors.InvalidHttpResponse,
             self.get_response, _invalid_response)

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-07-28 16:05:23 +0000
+++ bzrlib/transport/http/_pycurl.py	2006-08-31 21:34:20 +0000
@@ -174,7 +174,7 @@
         data.seek(0)
 
         code = curl.getinfo(pycurl.HTTP_CODE)
-        # mutter('url: %s header:\n%s', abspath, header.getvalue())
+        # mutter('header:\n%r', header.getvalue())
         headers = _extract_headers(header.getvalue(), abspath)
         # handle_response will raise NoSuchFile, etc based on the response code
         return code, response.handle_response(abspath, code, headers, data)

=== modified file 'bzrlib/transport/http/response.py'
--- bzrlib/transport/http/response.py	2006-07-18 20:27:50 +0000
+++ bzrlib/transport/http/response.py	2006-09-01 17:05:01 +0000
@@ -60,6 +60,8 @@
                                  self._ent_start, self._ent_end,
                                  self._data_start)
 
+    __repr__ = __str__
+
 
 class RangeFile(object):
     """File-like object that allow access to partial available data.
@@ -96,6 +98,8 @@
         i = bisect(self._ranges, self._pos) - 1
 
         if i < 0 or self._pos > self._ranges[i]._ent_end:
+            mutter('Bisect for pos: %s failed. Found offset: %d, ranges:%s',
+                   self._pos, i, self._ranges)
             raise errors.InvalidRange(self._path, self._pos)
 
         r = self._ranges[i]
@@ -134,7 +138,7 @@
 
     # TODO: jam 20060706 Consider compiling these regexes on demand
     _CONTENT_RANGE_RE = re.compile(
-        '\s*([^\s]+)\s+([0-9]+)-([0-9]+)/([0-9]+)\s*$')
+        r'\s*([^\s]+)\s+([0-9]+)-([0-9]+)/([0-9]+)\s*$')
 
     def __init__(self, path, content_range, input_file):
         # mutter("parsing 206 non-multipart response for %s", path)
@@ -175,7 +179,7 @@
     """A multi-range HTTP response."""
     
     _CONTENT_TYPE_RE = re.compile(
-        '^\s*multipart/byteranges\s*;\s*boundary\s*=\s*(.*?)\s*$')
+        r'^\s*multipart/byteranges\s*;\s*boundary\s*=\s*("?)([^"]*?)\1\s*$')
     
     # Start with --<boundary>\r\n
     # and ignore all headers ending in \r\n
@@ -195,6 +199,7 @@
         RangeFile.__init__(self, path, input_file)
 
         self.boundary_regex = self._parse_boundary(content_type, path)
+        # mutter('response:\n%r', self._data)
 
         for match in self.boundary_regex.finditer(self._data):
             ent_start, ent_end = HttpRangeResponse._parse_range(match.group(1),
@@ -216,7 +221,7 @@
             raise errors.InvalidHttpContentType(path, ctype,
                     "Expected multipart/byteranges with boundary")
 
-        boundary = match.group(1)
+        boundary = match.group(2)
         # mutter('multipart boundary is %s', boundary)
         pattern = HttpMultipartRangeResponse._BOUNDARY_PATT
         return re.compile(pattern % re.escape(boundary),

